#
# compute scores with ML from paired comparisons
#
# Author:  Christoph Dalitz
# Date:    24.06.2016
# License: GPL v2
# modified: Jens Wilberg, 18.09.2017

# Result of Wordrating
wordpairs <- read.csv("result.csv", sep = ";", stringsAsFactor = FALSE)
# List of words with already known scores
scores.all <- read.csv("wordrating/data/lexicon-scores.csv", row.names = 1,
                       stringsAsFactors = FALSE)
mm.all <- scores.all$norm
names(mm.all) <- row.names(scores.all)
t <- mm.all[names(mm.all) == "t"]
mm.all <- mm.all[names(mm.all) != "t"]
words <- unique(wordpairs$Wort1)

n.words <- length(words)

# distribution function
sdnorm <- 1 / sqrt(3)
scalelogis <- 1 / pi
F <- function(x) {
  return (pnorm(x, sd = sdnorm))
}
Finv <- function(x) {
  return (qnorm(x, sd = sdnorm))
}
Fprime <- function(x) {
  return (dnorm(x, sd = sdnorm))
}

# momentequation
# r  = the unknown score
# q  = scores of other words
# y  = game results (0 = win, 1 = tie, 2 = loss)
# t  = draw width
momentequation <- function(r, q, y, t) {
  rq <- r - q
  S <- sum(2 - y) / 2.0
  eq <- sum(F(rq)) - S
  #eq <- sum(F(rq - t)) + 0.5*sum(F(rq + t) - F(rq - t)) - S
  return (eq)
}

# compute scores
mm.scores <- rep(0, n.words)
names(mm.scores) <- as.character(words)
y <- rep(0, nrow(wordpairs))
y[wordpairs$Unentschieden == 1] <- 1
y[wordpairs$Sieg2 == 1] <- 2
for (word in words){
  q <- mm.all[wordpairs[wordpairs$Wort1 == word, ]$Wort2]
  yy <- y[wordpairs$Wort1 == word]
  # Check if word wins or looses all comprehension
  if (all(yy == 0)){
    # if all wins, the score is set to max + draw-width
    erg <- max(mm.all) + t
  }else if (all(yy == 2)){
    # if all looses, the score is set to min - draw-width
    erg <- min(mm.all) - t
  }else{
    res <- uniroot(momentequation, c(-5, 5), q = q, y = yy, t = t)
    erg <- res$root
  }
  mm.scores[word] <- erg
}

scores <- data.frame(x = mm.scores)


names(scores) <- c("norm")
write.csv(scores, file = "scores.csv")
